/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Analysis code of classical MC and MD simulations                    //
//                                                                     //
// Burkhard Militzer                               Washington 12-31-06 //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef _ANALYSIS_
#define _ANALYSIS_

#include "Averages.h"

class Analysis {
 public:
  ScalarAverage blockEnergy;
  ScalarAverage totalEnergy;
  ScalarAverage blockDisp;
  ScalarAverage totalDisp;
  const int blockMin;

  // let the main program decide to pass in K_total or K/N
  // P0 is the pressure for noninteracting system
  Analysis(const int blockMin_): 
    blockMin(blockMin_) {
    blockEnergy.SetLong();
    totalEnergy.SetLong();
  }

  void StartBlock(const int ib) {
    blockEnergy.SetToZero();
    blockDisp.SetToZero();
  }
  void EndBlock(const int ib, const bool print=true) {
    if (ib>=blockMin) {
      totalEnergy.Add  (blockEnergy.GetAverage());
      totalDisp.Add    (blockDisp.GetAverage());
    }
    if (print) cout << *this;
  }
  void EndSimulation(const bool print=true) {
    blockEnergy.SetLong();
    totalEnergy.SetLong();
    blockDisp.SetLong();
    totalDisp.SetLong();
    if (print) cout << *this;
  }

  void AddTotalEnergy(const double e) {
    blockEnergy.Add(e);
  }
  void AddDisplacement(const double d) {
    blockDisp.Add(d);
  }

  static string PFS(const ScalarAverage & s, const Form & f, const Form & fe) {
    ostringstream os;
    os << f(s.GetAverage()) << " +- " << fe(s.GetErrorBar()) << " ";
    return os.str();
  }

  static string PFS(const ScalarAverage & s, const Form & f) {
    return PFS(s,f,f);
  }

  void PrintBlockEnergy() {
    Form ff(fix,14,8);
    Form fe(fix,11,8);
    cout << " Energy= " << PFS(blockEnergy,ff,fe);
  }

  friend ostream & operator<<(ostream &os, const Analysis & a ) {
    Form ff(fix,14,8);
    Form fe(fix,11,8);
    //    os << endl;
    //  os << "DR : block: " << PFS(a.blockDisp      ,ff,fe) << " total: " << PFS(a.totalDisp      ,ff,fe) << endl;
    os << " Energy : block: " << PFS(a.blockEnergy,ff,fe) << " total: " << PFS(a.totalEnergy,ff,fe) << endl;
    //    os << endl;
    return os;
  }

};

#endif // _ANALYSIS_
